/* 
 * Copyright (c) 2009, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== dman3_api.c ========
 *  DMA Resource manager and utilities for granting logical channel
 *  resources to algorithms.
 */

#include <xdc/std.h>

#ifdef xdc_target__isaCompatible_64P
#include <ti/bios/include/mem.h>
#include <ti/sdo/fc/utils/dbc.h>
#else
#include "bios_rts.h"
#endif

#include <string.h> /* For memset() */

#include "rmm.h"
#include "_rmm.h"
#include "dman3.h"
#include "dman3_impl.h"

#include <ti/sdo/utils/trace/gt.h>
GT_Mask ti_sdo_fc_dman3_GTMask;

#ifdef xdc_target__os_Linux
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <unistd.h>
#include <fcntl.h>

#define SHMSIZE 0x4000      /* Size of shared memory chunk to allocate */

#endif

/*
 *  QCHMAP: Queue to channel map register which specifies which parameter
 *  entry in bits 13-5 and the trigger word (which is word 7 in ACPY3
 *  implementation), in bits 4-2.
 */
//#define QCHMAP ((volatile Uns *)(0x01c00200))

// TODO: These 2 defines will go away with EDMA3 regs structures
#define QCHMAPOFFSET 0x200
#define PARAMOFFSET 0x4000
#define PARAMENTRYSIZE 8  /* Number of words in PaRam entry */

/* Byte offsets of QDMA registers */
#define QDMAQNUMOFFSET  0x0260
#define QUETCMAPOFFSET  0x0280
#define QUEPRIOFFSET    0x0284
#define QRAE0OFFSET     0x0380
#define QEESROFFSET     0x108C

/* Sizes of EDMA3 register and PaRam regions to map */
#define EDMA3REGSIZE 0x2000
#define PARAMREGSIZE 0x800

/* Align on 4-byte boundary */
#define ALIGN32(x) ((((UInt32)(x) + 3) / 4) * 4)

/* For putting offsets into shared data area rather than addresses */
#define OFFSET(base, address) ((UInt32)address - (UInt32)base)

static int      refCount = 0;   /* Process reference count */
_DMAN3_Data *_DMAN3_data;


#ifdef xdc_target__os_Linux

static Void freeDman3Obj();
static Bool getDman3Obj();
static Void unMapShm();

#ifndef _DMAN3_CPUCOPY_
static int      memfd;          /* File discriptor for "/dev/mem" */
#endif

static Int      shmId;          /* Shared memory id */
#endif /* xdc_target__os_Linux */

#ifdef _MULTI_PROCESS_
LockMP_Handle _DMAN3_lock = NULL;
#endif

static Void    *shmBase = NULL; /* Base address of shared memory chunk */
static Void    *shmFreeBase;    /* Next available location in shared memory */
static UInt32   shmSize;
static Int      heapInt;
static Int      heapExt;

UInt32          *_DMAN3_paRamAddr = NULL;
UInt32          *_DMAN3_edma3Addr = NULL;
UInt32          _DMAN3_nullPaRam = 0x0;


/*
 *  Functions for allocating and freeing from the shared memory block.
 */
static Void *shmAlloc(Int segid, UInt32 size, UInt32 align);
static Bool shmFree(Int segid, Void *addr, UInt32 size);

static Int getNumAssignedTccs(UInt32 mask32b, UInt32 * maxTcc);
static Void updateTccAllocationTable(UInt32 mask32b, Int base);
static Void initQdmaConfiguration();
static Bool checkDMAN3Configuration();
static Int initializeDMAN3(); 
static Int finalizeDMAN3(); 



#include "dman3.h"
#include "dman3_impl.h"
#include "rmm.h"


#ifdef xdc_target__isaCompatible_64P

#include <ti/sdo/fc/utils/dbc.h>
#include <ti/bios/include/mem.h>

#else
#include "bios_rts.h"
#endif

#include <ti/xdais/idma3.h>
#include <ti/sdo/fc/utils/gtinfra/gt_dais.h>

#ifdef __TI_COMPILER_VERSION__

#pragma CODE_SECTION(DMAN3_grantDmaChannels, ".text:DMAN3_grantDmaChannels");
#pragma CODE_SECTION(DMAN3_releaseDmaChannels,".text:DMAN3_releaseDmaChannels");

#pragma CODE_SECTION(DMAN3_createChannels, ".text:DMAN3_createChannels");
#pragma CODE_SECTION(DMAN3_freeChannels, ".text:DMAN3_freeChannels");

#endif

extern Uint32 *_DMAN3_paRamAddr;


/*
 *  Number of words and MAUs in one PaRam entry. PARAMSIZEMAUS should be
 *  sizeof(ACPY3_PaRamRegs), but use #define since we can't include ACPY3
 *  header file here.
 */
#define PARAMSIZE 8
#define PARAMSIZEMAUS (PARAMSIZE * sizeof(Int))

/*
 *  ======== DMAN3_ChannelObj ========
 *  Structure to store a handle to the DMAN3 context object that
 *  was used to create the IDMA3 channel.
 */
typedef struct DMAN3_ChannelObj {
    IDMA3_Obj           idmaChan;
    DMAN3_Context       context;
    Int                 segEnv;  /* segment id where env is allocated */
    Uint16              sizeEnv; /* size of env */
} DMAN3_ChannelObj, *DMAN3_Channel;


static Int  createChannels(DMAN3_Context context, IALG_Handle alg, Int groupId,
                           IDMA3_ChannelRec dmaTab[], Int numChans);
static Int  createContext(DMAN3_Context *pContext, Int groupId, Int nTccs,
                       Int nPaRams);
static Void freeChannels(IDMA3_ChannelRec dmaTab[], Int numChan);
static Void freeContext(DMAN3_Context context);
static Void freeScratch(Int groupId, IALG_MemRec memTab[], Int numRecs);
static Int  getContext(DMAN3_Context *pContext, Int groupId, Int nTccs,
                       Int nPaRams);
static Bool initHandle(IDMA3_Handle handle, IDMA3_ChannelRec *dmaTab,
                       Uint8 *tccList, Uns qdmaChannel,
                       Int16 envSize, Void *env, Void *scratchEnv, Int envSeg,
                       DMAN3_Context context);


/*
 *  ======== DMAN3_createChannels ========
 *  Allocate and initialize memory for channel handle.
 */
Int DMAN3_createChannels(Int groupId, IDMA3_ChannelRec dmaTab[], Int numChans)
{
    Int             i, j;
    DMAN3_Context   context;
    Int             maxWaits = 0;
    Int             maxTransfers = 0;
    Int             status = DMAN3_SOK;
    Uns             * gtPaRamAddr = NULL;

    DBC_require(groupId < DMAN3_MAXGROUPS);
    DBC_require(numChans >= 0);
    DBC_require((dmaTab != NULL) || (numChans == 0));
    GT_assert(ti_sdo_fc_dman3_GTMask, groupId < DMAN3_MAXGROUPS);
    GT_assert(ti_sdo_fc_dman3_GTMask, numChans >= 0);
    GT_assert(ti_sdo_fc_dman3_GTMask, (dmaTab != NULL) || (numChans == 0));

    GT_3trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_createChannels> Enter "
            "(groupId=%d, dmaTab=0x%x, numChans=%d\n", groupId, dmaTab,
            numChans);

    if (0 == refCount) {
        status = initializeDMAN3();
        if (status != DMAN3_SOK) {
            return (status);
        }
    }

    /*
     *  Get the maximum number of waits and transfers to see how many
     *  PaRams and TCCs are needed.
     */
    for (i = 0; i < numChans; i++) {
        DBC_require(dmaTab[i].numWaits >= 0);
        DBC_require(dmaTab[i].numTransfers > 0);
        GT_assert(ti_sdo_fc_dman3_GTMask, dmaTab[i].numWaits >= 0);
        GT_assert(ti_sdo_fc_dman3_GTMask, dmaTab[i].numTransfers > 0);

        dmaTab[i].handle = NULL;
        maxWaits += dmaTab[i].numWaits;
        maxTransfers += dmaTab[i].numTransfers;
    }

    GT_3trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "DMAN3_createChannels> "
            "Total Qdma channels requested %d, PaRams requested %d, "
            "Tccs requested %d\n", numChans, maxTransfers, maxWaits);

#ifdef _MULTI_PROCESS_
    /* Enter critical section */
    LockMP_acquire(_DMAN3_lock);
#endif
    /* Get the context for this group ID, or create a new one. */
    status = getContext(&context, groupId, maxWaits, maxTransfers);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "DMAN3_createChannels> "
            "Context 0x%x\n", context );

    /* Now create the channels */
    if (status == DMAN3_SOK) {
        status = createChannels(context, NULL, groupId, dmaTab, numChans);
    }

    if (status == DMAN3_SOK) {

        for (i = 0; i < numChans; i++) {

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                    "DMAN3_createChannels> Assigned Qdma: %d\n",
                    dmaTab[i].handle->qdmaChan);

            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                    "DMAN3_createChannels> Assigned PaRam(s):\n");

            gtPaRamAddr = dmaTab[i].handle->paRamAddr;

            GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                    "DMAN3_createChannels> dmaTab[%d].handle's "
                    "paRamAddr 0x%x\n", i, gtPaRamAddr);

            for(j = 0; j < dmaTab[i].handle->numPaRams; j++) {

                GT_2trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                        "DMAN3_createChannels> %d (Addr 0x%x)\n",
                        ((Uns)gtPaRamAddr - (Uns)_DMAN3_paRamAddr) /
                        PARAMSIZEMAUS,
                        gtPaRamAddr);

                gtPaRamAddr -= (PARAMSIZE);
            }

            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                    "DMAN3_createChannels> Assigned Tcc(s):\n");

            for(j = 0; j < dmaTab[i].handle->numTccs; j++) {

                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                        "DMAN3_createChannels> %d\n",
                        dmaTab[i].handle->tccTable[j]);
            }
        }

    }

#ifdef _MULTI_PROCESS_
    /* Exit critical section */
    LockMP_release(_DMAN3_lock);
#endif

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_createChannels> Exit "
            "(returnVal=%d)\n",status);

    return (status);
}

/*
 *  ======== DMAN3_grantDmaChannels ========
 *  Add an algorithm to the DMA Manager.  The DMA Manager will grant DMA
 *  resources to the algorithm as a result.  This function is called when
 *  initializing an algorithm instance.
 */
/* ARGSUSED - this line tells the compiler not to warn about unused args. */
Int DMAN3_grantDmaChannels(Int groupId, IALG_Handle algHandle[],
        IDMA3_Fxns *dmaFxnsArray[], Int numAlgs)
{
    Uns              numChans;
    Int              i, j, k;
    IALG_Handle      alg;
    IDMA3_Fxns      *dmaFxns;
    IDMA3_ChannelRec *dmaTab;
    Int              maxWaits = 0;
    Int              maxTransfers = 0;
    DMAN3_Context    context;
    Int              status = DMAN3_EFAIL;
    Uns             *gtPaRamAddr = NULL;

    DBC_require(algHandle != NULL);
    DBC_require(dmaFxnsArray != NULL);
    DBC_require(numAlgs > 0);
    DBC_require(groupId < DMAN3_MAXGROUPS);
    GT_assert(ti_sdo_fc_dman3_GTMask, algHandle != NULL);
    GT_assert(ti_sdo_fc_dman3_GTMask, dmaFxnsArray != NULL);
    GT_assert(ti_sdo_fc_dman3_GTMask, numAlgs > 0);
    GT_assert(ti_sdo_fc_dman3_GTMask, groupId < DMAN3_MAXGROUPS);

    GT_4trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_grantDmaChannels> "
            "Enter (groupId=%d, algHandle=0x%x, dmaFxns=0x%x, numAlgs=%d)\n",
            groupId, algHandle, dmaFxnsArray, numAlgs);

    if (0 == refCount) {
        status = initializeDMAN3();
        if (status != DMAN3_SOK) {
            return (status);
        }
    }

    /* use dynamic allocation instead of stack to avoid large array on stack */
    dmaTab = (IDMA3_ChannelRec *)MEM_calloc(_DMAN3_heapExt,
        sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs, sizeof(Int));

    if (dmaTab == MEM_ILLEGAL) {

        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                "DMAN3_grantDmaChannels> Exit (status=DMAN3_EOUTOFMEMORY)\n");
        return (DMAN3_EOUTOFMEMORY);
    }

    /*
     *  Determine the maximum number of TCCs and PaRams that will be needed
     *  by this group of algorithms.
     */
    for (i = 0; i < numAlgs; i++) {

        alg = algHandle[i];
        dmaFxns = dmaFxnsArray[i];

        DBC_require(alg != NULL);
        DBC_require(dmaFxns != NULL);
        GT_assert(ti_sdo_fc_dman3_GTMask, alg != NULL);
        GT_assert(ti_sdo_fc_dman3_GTMask, dmaFxns != NULL);

        /*  alg and idma2 fxns must be from same implementation */
        DBC_require(dmaFxns->implementationId == alg->fxns->implementationId);
        GT_assert(ti_sdo_fc_dman3_GTMask,
                dmaFxns->implementationId == alg->fxns->implementationId);

        /*
         * Verify that algo is not using two many buffers,
         * in release build trust that contract is respected
         */

        numChans = dmaFxns->dmaGetChannels(alg, dmaTab);


        /* TODO:L  should additionally dump dmaTab? */
        for (j = 0; j < numChans; j++) {

            dmaTab[j].handle = NULL;
            DBC_require(dmaTab[j].numWaits >= 0);
            DBC_require(dmaTab[j].numTransfers > 0);
            GT_assert(ti_sdo_fc_dman3_GTMask, dmaTab[j].numWaits >= 0);
            GT_assert(ti_sdo_fc_dman3_GTMask, dmaTab[j].numTransfers > 0);

            maxWaits += dmaTab[j].numWaits;
            maxTransfers += dmaTab[j].numTransfers;
        }


        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                "DMAN3_grantDmaChannels> Alg 0x%x:\n", alg);

        GT_3trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "DMAN3_grantDmaChannels> "
                "Total Channels requested %d, PaRams requested %d, "
                "Tccs requested %d\n", numChans, maxTransfers, maxWaits);

        /*
         * Note, we check tccs here, but can't check PaRams because it's a
         * more complex check... and it's done in getContext().
         */
        if (maxWaits > _DMAN3_numAssignedTccs) {

            MEM_free(_DMAN3_heapExt, dmaTab,
                    sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs);
            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                    "DMAN3_grantDmaChannels> Exit (status=EOUTOFTCCS)\n");
            return (DMAN3_EOUTOFTCCS);
        }
    }

#ifdef _MULTI_PROCESS_
    /* Enter critical section */
    LockMP_acquire(_DMAN3_lock);
#endif

    /* Get the context for this group, or create a new one. */
    status = getContext(&context, groupId, maxWaits, maxTransfers);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "DMAN3_grantDmaChannels> "
            "Context 0x%x\n", context);

    if (status == DMAN3_SOK) {
        for (i = 0; i < numAlgs; i++) {
            alg = algHandle[i];
            dmaFxns = dmaFxnsArray[i];

            DBC_require(alg != NULL);
            DBC_require(dmaFxns != NULL);
            GT_assert(ti_sdo_fc_dman3_GTMask, alg != NULL);
            GT_assert(ti_sdo_fc_dman3_GTMask, dmaFxns != NULL);

            /*  alg and idma3 fxns must be from same implementation */
            DBC_require(dmaFxns->implementationId ==
                    alg->fxns->implementationId);
            GT_assert(ti_sdo_fc_dman3_GTMask,
                    dmaFxns->implementationId == alg->fxns->implementationId);

            numChans = dmaFxns->dmaGetChannels(alg, dmaTab);

            status = createChannels(context, alg, groupId, dmaTab, numChans);

            if ((status == DMAN3_SOK)) {

                if ((status = dmaFxns->dmaInit(alg, dmaTab)) != IALG_EOK) {
                    /* If dmaInit fails, free channel resources for alg[i] */
                    freeChannels(dmaTab, numChans);

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                            "DMAN3_grantDmaChannels> Alg 0x%x dmaInit failed, "
                            "freeing channel resources\n", alg);

                    status = DMAN3_EFAIL;
                }
                else {

                    for (k = 0; k < numChans; k++) {

                        /* Print out trace information */
                        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                                "DMAN3_createChannels> Assigned Qdma: %d\n",
                                dmaTab[k].handle->qdmaChan);

                        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                                "DMAN3_createChannels> Assigned PaRam(s):\n");

                        gtPaRamAddr = dmaTab[k].handle->paRamAddr;

                        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                                "DMAN3_createChannels> dmaTab[%d].handle's "
                                "paRamAddr 0x%x\n", k, gtPaRamAddr);

                        for (j = 0; j < dmaTab[k].handle->numPaRams; j++) {

                            GT_2trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                                    "DMAN3_createChannels> %d (Addr 0x%x)\n",
                                    ((Uns)gtPaRamAddr - (Uns)_DMAN3_paRamAddr) /
                                    PARAMSIZEMAUS,
                                    gtPaRamAddr);

                            gtPaRamAddr -= (PARAMSIZE);
                        }

                        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                                "DMAN3_createChannels> Assigned Tcc(s):\n");

                        for(j = 0; j < dmaTab[k].handle->numTccs; j++) {

                            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                                    "DMAN3_createChannels> %d\n",
                                    dmaTab[k].handle->tccTable[j]);
                        }
                    }
                }
            }

            if (status != DMAN3_SOK) {
                /* Free channels allocated for alg[0],...,alg[i-1] */
                for (j = 0; j < i; j++) {
                    alg = algHandle[j];
                    dmaFxns = dmaFxnsArray[j];
                    numChans = dmaFxns->dmaGetChannels(alg, dmaTab);

                    /* freeChannels frees context if refCount goes to 0 */
                    freeChannels(dmaTab, numChans);

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                            "DMAN3_grantDmaChannels> Alg 0x%x Freeing channel "
                            "resources\n", alg);

                }
                break;
            }
        }
    }

#ifdef _MULTI_PROCESS_
        /* Exit critical section */
        LockMP_release(_DMAN3_lock);
#endif

    MEM_free(_DMAN3_heapExt, dmaTab,
        sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_grantDmaChannels> Exit"
            " (status=%d)\n",status);

    return (status);
}

/*
 *  ======== DMAN3_freeChannels ========
 *  Free memory for channel handles in dmaTab[].
 */
/* ARGSUSED - this line tells the compiler not to warn about unused args. */
Int DMAN3_freeChannels(IDMA3_Handle channelTab[], Int numChan)
{
    Int             i;
    IDMA3_Handle    chan;
    DMAN3_Context   context;
    DMAN3_Channel   dmanChan;
    Void           *scratchEnv;
    Int             status = DMAN3_SOK;
#if DBC_ASSERTS
    Bool            fRet;
#endif

    DBC_require(numChan >= 0);
    DBC_require((channelTab != NULL) || (numChan == 0));
    GT_assert(ti_sdo_fc_dman3_GTMask, numChan >= 0);
    GT_assert(ti_sdo_fc_dman3_GTMask, (channelTab != NULL) || (numChan == 0));

    GT_2trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_freeChannels> Enter "
            " (channelTab=0x%x, numChan=%d)\n",channelTab, numChan);

    for (i = 0; i < numChan; i++) {

        chan = channelTab[i];
        DBC_require(chan != NULL);
        GT_assert(ti_sdo_fc_dman3_GTMask, chan != NULL);

        dmanChan = (DMAN3_Channel)chan;
        context = dmanChan->context;
        DBC_assert(context != NULL);
        GT_assert(ti_sdo_fc_dman3_GTMask, context != NULL);

        /* De-init the environment */
        if (chan->protocol && chan->protocol->deInitHandle) {

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "DMAN3_freeChannels> "
                    "Calling deinit on protocol on Channel %d\n", i);

#if DBC_ASSERTS
            fRet = chan->protocol->deInitHandle(chan);
            DBC_assert(fRet);
            GT_assert(ti_sdo_fc_dman3_GTMask, fRet);
#else
            chan->protocol->deInitHandle(chan);
#endif
        }

        /* Free 'env' buffer, but only if it has been allocated. */
        if ((dmanChan->sizeEnv > 0) && (chan->env != NULL)) {

            /*
             *  Free scratchEnv if it has been allocated. The deInit() function
             *  must put the scratch pointer back at the beginning of the env
             *  address.
             */
            scratchEnv = (Void *)(*(Uint32 *)(chan->env));

            if ((scratchEnv != NULL) && (_DMAN3_freeScratch != NULL)) {

                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                        "DMAN3_freeChannels> Free scratchEnv 0x%x\n",
                        scratchEnv);

                (_DMAN3_freeScratch)(context->scratchId, scratchEnv,
                        dmanChan->sizeEnv);
            }
            /* Get the memory segment for the environment */
#if DBC_ASSERTS
            fRet = MEM_free(dmanChan->segEnv, chan->env, dmanChan->sizeEnv);
            DBC_assert(fRet);
#else
            MEM_free(dmanChan->segEnv, chan->env, dmanChan->sizeEnv);
#endif
        }

        /* Free DMAN3 channel object memory */
#if DBC_ASSERTS
        fRet = MEM_free(_DMAN3_heapIdma3, chan, sizeof(DMAN3_ChannelObj));
        DBC_assert(fRet);
#else
        MEM_free(_DMAN3_heapIdma3, chan, sizeof(DMAN3_ChannelObj));
#endif

#ifdef _MULTI_PROCESS_
        /* Enter critical section: for non-64P, context is in shared memory */
        LockMP_acquire(_DMAN3_lock);
#endif
        context->refCount--;

        /* Free the context object if the refCount is 0 */
        if (context->refCount == 0) {
            freeContext(context);
        }
#ifdef _MULTI_PROCESS_
        /* Exit critical section */
        LockMP_release(_DMAN3_lock);
#endif
    }

    refCount -= numChan;

    if (0 == refCount) {
        status = finalizeDMAN3();
        if (status != DMAN3_SOK) {
            return (status);
        }
    }

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_freeChannels> Exit "
            "(status=DMAN3_SOK)\n");

    return (DMAN3_SOK);
}


/*
 *  ======== DMAN3_releaseDmaChannels ========
 *  Remove logical channel resources from an algorithm instance
 */
/* ARGSUSED - this line tells the compiler not to warn about unused args. */
Int DMAN3_releaseDmaChannels(IALG_Handle algs[], IDMA3_Fxns *dmaFxnsArray[],
        Int numAlgs)
{
    IALG_Handle         alg;
    IDMA3_Fxns         *dmaFxns;
    IDMA3_ChannelRec    *dmaTab;
    Uns                 nChans;
    Int                 i;

    DBC_require(algs != NULL);
    DBC_require(dmaFxnsArray != NULL);
    DBC_require(numAlgs > 0);
    GT_assert(ti_sdo_fc_dman3_GTMask, algs != NULL);
    GT_assert(ti_sdo_fc_dman3_GTMask, dmaFxnsArray != NULL);
    GT_assert(ti_sdo_fc_dman3_GTMask, numAlgs > 0);

    GT_3trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_releaseDmaChannels> "
            "Enter (algs=0x%x, dmaFxnsArray=0x%x, numAlgs=%d)\n", algs,
            dmaFxnsArray, numAlgs);

    /* use dynamic allocation instead of stack to avoid large array on stack */
    dmaTab = (IDMA3_ChannelRec *)MEM_calloc(_DMAN3_heapExt,
        sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs, sizeof(Int));

    if (dmaTab == MEM_ILLEGAL) {
        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                "DMAN3_releaseDmaChannels> Exit (status=DMAN3_EOUTOFMEMORY\n");
        return (DMAN3_EOUTOFMEMORY);
    }

    for (i = 0; i < numAlgs; i++) {
        dmaFxns = dmaFxnsArray[i];
        alg = algs[i];

        DBC_require(dmaFxns->implementationId == alg->fxns->implementationId);
        GT_assert(ti_sdo_fc_dman3_GTMask,
                dmaFxns->implementationId == alg->fxns->implementationId);

        /*
         * In debug build verify that algo is not using two many buffers,
         * in release build trust that contract is respected
         */

        nChans = dmaFxns->dmaGetChannels(alg, dmaTab);

        if (nChans > _DMAN3_numAssignedTccs) {
            MEM_free(_DMAN3_heapExt, dmaTab,
                    sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs);
            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                    "DMAN3_releaseDmaChannels> Exit "
                    "(status=DMAN3_EOUTOFTCCS\n");
            return (DMAN3_EOUTOFTCCS);
        }

        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                "DMAN3_releaseDmaChannels> Free %d channels\n", nChans);

        freeChannels(dmaTab, nChans);

    }

    MEM_free(_DMAN3_heapExt, dmaTab,
        sizeof(IDMA3_ChannelRec) * _DMAN3_numAssignedTccs);

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_releaseDmaChannels> "
            "Exit (status=DMAN3_SOK)\n");

    return (DMAN3_SOK);
}


/*
 *  ======== createChannels ========
 */
static Int createChannels(DMAN3_Context context, IALG_Handle alg, Int groupId,
    IDMA3_ChannelRec dmaTab[], Int numChans)
{
    Void           *env;
    Void           *scratchEnv;
    Int             envSize;
    Int             seg;
    Int             align;
    IDMA3_MemRec    memRec;
    IALG_MemRec    *memTab = NULL;
    IALG_MemAttrs   memType;
    Int             i, j;
    Uns             qDmaChan;
    Uint8          *tccTablePtr;
    Uns            *paRamAddr;
    Bool            fRet;
    Int             status = DMAN3_SOK;
    Uns             * gtPaRamAddr = NULL;

    DBC_assert(dmaTab != NULL);

    GT_5trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createChannels> Enter "
            "(context=0x%x, alg=0x%x, groupId=%d, dmaTab=0x%x, numChans=%d)\n",
            context, alg, groupId, dmaTab, numChans);

    /*
     *  Allocate array of memTab to be used if env can be allocated from
     *  shared scratch memory. Scratch memory for the alg has to be allocated
     *  all at once, so we need to collect the env memory requirements for
     *  each of the alg's channels. All sizes are initialized to 0 here.
     */
    memTab = (IALG_MemRec *)MEM_calloc(_DMAN3_heapExt,
            sizeof(IALG_MemRec) * numChans, sizeof(Int));

    if (memTab == MEM_ILLEGAL) {
        status = DMAN3_EOUTOFMEMORY;
    }
    else {
        /* Type of memory for env allocated in internal memory */
        memType = (_DMAN3_allocScratch != NULL) ? IALG_SCRATCH : IALG_PERSIST;

        /* Fill in memTab with each channels' env memory requirements */
        for (i = 0; i < numChans; i++) {

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Channel %d\n", i);

            memTab[i].attrs = IALG_PERSIST;

            if (dmaTab[i].protocol && dmaTab[i].protocol->getEnvMemRec) {

                dmaTab[i].protocol->getEnvMemRec(&dmaTab[i], &memRec);
                memTab[i].size = memRec.size;
                memTab[i].alignment = memRec.align;

                /*
                 *  If memRec.memType = IDMA3_INTERNAL, we arbitrarily set the
                 *  space to IALG_DARAM0.
                 */
                if (memRec.memType == IDMA3_INTERNAL) {
                    memTab[i].space = IALG_DARAM0;
                    memTab[i].attrs = memType;
                }
                else {
                    memTab[i].space = IALG_EXTERNAL;
                }

                GT_5trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "createChannels>"
                        " Env Requested memTab[%d]: size=0x%x, align=0x%x, "
                        "space=%s, attrs=%s\n",i, memTab[i].size,
                        memTab[i].alignment, gt_MemAttrs(memTab[i].attrs),
                        gt_MemSpace(memTab[i].space));
            }
        }

        /* Allocate scratch env */
        if ((_DMAN3_allocScratch != NULL) &&
                !_DMAN3_allocScratch(alg, groupId, memTab, numChans)) {
            /*
             *  Don't fail if allocation in scratch memory failed. We will
             *  try to allocate in persistent memory.
             */
            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Scratch Allocation failed\n");

            for (i = 0; i < numChans; i++) {
                memTab[i].attrs = IALG_PERSIST;
                memTab[i].base = MEM_ILLEGAL;
            }
        }
                else {
                        if ((_DMAN3_allocScratch != NULL) && (dmaTab[i].protocol)) {

                for (i = 0; i < numChans; i++) {

                                    GT_6trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, 
                            "createChannels> Scratch Env Allocated memTab[%d]: "
                            "size=0x%x, align=0x%x, space=%s, attrs=%s, "
                            "base=0x%x\n",i, memTab[i].size,
                                        memTab[i].alignment, gt_MemAttrs(memTab[i].attrs),
                                        gt_MemSpace(memTab[i].space), memTab[i].base);
                                }
                        }
                }
    }

    if (status != DMAN3_SOK) {
        /* Check the context refCount to see if it should be freed. */
        if (context->refCount == 0) {
            freeContext(context);
        }
        return (status);
    }

    /* Now we can start creating channels */
    for (i = 0; i < numChans; i++) {

        envSize = memTab[i].size;
        seg = (memTab[i].space == IALG_EXTERNAL) ? _DMAN3_heapExt :
                                                   _DMAN3_heapInt;
        align = memTab[i].alignment;
        scratchEnv = memTab[i].base; /* NULL unless allocated in scratch */

        GT_4trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                "Creating channel %d, size 0x%x, align 0x%x, scratchEnv 0x%x\n",
                i, envSize, align, scratchEnv);

        /*
         *  Align on word boundary to simplify accesses of IDMA3_Obj
         *  fields by assembly implementations of ACPY3. Allocate more
         *  than IDMA3_Obj, so we can store the context associated with
         *  this channel.
         */

        GT_4trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "createChannels> "
                "IDMA3 Object Requested mem[%d]: size=0x%x, align=0x%x "
                "heapId=%d\n", i, sizeof(DMAN3_ChannelObj), sizeof(Int),
                _DMAN3_heapIdma3);

        dmaTab[i].handle = (IDMA3_Obj *)MEM_calloc(_DMAN3_heapIdma3,
                sizeof(DMAN3_ChannelObj), sizeof(Int));

        if (dmaTab[i].handle == MEM_ILLEGAL) {

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Error allocating memory for channel %d\n", i);

            /*
             *  If we failed to create the first channel, then freeChannels()
             *  will not get called to free the context if its ref count is 0.
             *  This can only happen when i = 0.
             */
            if (context->refCount == 0) {

                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels>"
                        " Freeing context 0x%x\n",context);

                freeContext(context);
            }

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Freeing channels %d\n", i);

            /* Free up the channels that were already allocated. */
            freeChannels(dmaTab, i);

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Freeing scratch memory of group %d\n", groupId);

            /* Free scratch memory not freed in call to freeChannels(). */
            freeScratch(groupId, memTab + i, numChans - i);

            status = DMAN3_EOUTOFMEMORY;
            break;
        }
        else {
            /* Assert that Physical DMA resources can be provided */
            DBC_assert(context->nextFreeTccIdx + dmaTab[i].numWaits <=
                    context->tccBase + context->numTccs);
            DBC_assert(context->numAssignedPaRams + dmaTab[i].numTransfers <=
                    context->numPaRams);

            GT_5trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "createChannels> "
                    "IDMA3 Object Allocated mem[%d]: size=0x%x, align=0x%x, "
                    "heapId=%d, base=0x%x\n", i, sizeof(DMAN3_ChannelObj), 
                    sizeof(Int), _DMAN3_heapIdma3, dmaTab[i].handle);

            /* Assign QDMA Channel in a roundrobin fashion */
            qDmaChan = DMAN3_PARAMS.qdmaChannels[context->nextQdmaChannel++
                    % DMAN3_PARAMS.numQdmaChannels];

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                    "Assigning qdmaChannel %d\n",qDmaChan);

            /* Assign TCCs sequentially from a free list */
            tccTablePtr = &_DMAN3_tccAllocationTable[context->nextFreeTccIdx];
            context->nextFreeTccIdx += dmaTab[i].numWaits;

            /* Allocate the 'env' buffer only when requested (i.e. > 0) */
            if ((envSize > 0)) {

                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels>"
                        " Env buffer of size 0x%x requested\n",envSize);

                /*
                 *  If we have env allocated in scratch, we need to allocate
                 *  a persistent copy in external memory.
                 */
                if (scratchEnv != NULL) {
                    seg = _DMAN3_heapExt;
                }

                env = MEM_calloc(seg, envSize, align);
            }
            else {

                env = NULL;
            }

            if ((envSize <= 0) || (env != MEM_ILLEGAL)) {
                //TODO: need for atomicity when reading/writing context? (nd)
                context->refCount++;

                if (envSize > 0) {

                    GT_5trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS,
                            "createChannels> "
                            "Env Allocated memTab[%d]: size=0x%x, "
                            "align=0x%x, heapId=%d, base=0x%x\n", i, envSize,
                            align, seg, env);
                }

                GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels>"
                        " Initializing IDMA3_handle\n");

                /*
                 * Initialize the IDMA3_Handle handle with resource allocation
                 * & channel state
                 */
                fRet = initHandle(dmaTab[i].handle, &dmaTab[i], tccTablePtr,
                        qDmaChan, envSize, env, scratchEnv, seg, context);

                if (!fRet) {

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Error initializing IDMA3_handle, "
                            "freeChannels %d\n", i+1);

                    /* free the channels created so far, including this one */
                    freeChannels(dmaTab, i + 1);

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Free remaining scratch from "
                            "group %d\n", groupId);

                    /* Free remaining scratch */
                    freeScratch(groupId, memTab + i + 1, numChans - i - 1);
                    status = DMAN3_EFAIL;
                    break;
                }
                else {

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Assigned Qdma Channel(s):%d\n",
                            qDmaChan);

                    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Assigned PaRam(s):\n");

                    gtPaRamAddr = dmaTab[i].handle->paRamAddr;

                    GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> dmaTab[%d].handle's "
                            "paRamAddr 0x%x\n", i, gtPaRamAddr);

                    for(j = 0; j < dmaTab[i].handle->numPaRams; j++) {

                        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                                "createChannels> %d (Addr 0x%x)\n",
                                ((Uns)gtPaRamAddr - (Uns)_DMAN3_paRamAddr) /
                                PARAMSIZEMAUS,
                                gtPaRamAddr);

                        gtPaRamAddr -= (PARAMSIZE);
                    }

                    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Assigned Tcc(s):\n");

                    for (j = 0; j < dmaTab[i].handle->numTccs; j++) {
                        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                                "createChannels> %d\n",
                                dmaTab[i].handle->tccTable[j]);

                    }

                    /* Update paRamAddr for next channel */
                    paRamAddr = ((Uns *)context->nextFreePaRamAddr) +
                        (PARAMSIZE * dmaTab[i].numTransfers);

                    /* Set free ptr to next entry */
                    context->nextFreePaRamAddr = ((Uns *)paRamAddr);
                    context->numAssignedPaRams += dmaTab[i].numTransfers;

                    /* Reference count the channels */
                    refCount ++;
                }
            }
            else {

                GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createChannels> "
                        "Error allocating environment memory, freeing Channel "
                        "objects\n");

                /* Couldn't allocate env */
                MEM_free(_DMAN3_heapIdma3, dmaTab[i].handle,
                        sizeof(DMAN3_ChannelObj));

                /*
                 *  Check the context refCount to see if it should be freed
                 *  (ie, if this were the first channel we attempted to
                 *  create).
                 */
                if (context->refCount == 0) {

                    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                            "createChannels> Free context 0x%x\n",context);

                    freeContext(context);
                }

                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                        "createChannels> Free Channels %d\n", i);

                freeChannels(dmaTab, i);

                /* Free remaining scratch */
                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                        "createChannels> Free scratch from group %d\n",
                        groupId);

                freeScratch(groupId, memTab + i, numChans - i);
                status = DMAN3_EOUTOFMEMORY;
                break;
            }
        }
    }

    if (memTab) {
        MEM_free(_DMAN3_heapExt, memTab, sizeof(IALG_MemRec) * numChans);
    }

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createChannels> Exit "
            "(status=%d)\n",status);

    return (status);
}


/*
 *  ======== createContext ========
 */
static Int createContext(DMAN3_Context *pContext, Int groupId, Int nTccs,
        Int nPaRams)
{
    Int             maxTccs;
    Int             maxPaRams;
    Int             paRamBaseIndex;
    DMAN3_Context   context;
    Int             status = DMAN3_SOK;

    GT_4trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createContext> Enter "
            "(pContext=0x%x, groupId=%d, nTccs=%d, nPaRams=%d)\n", pContext,
            groupId, nTccs, nPaRams);

    *pContext = NULL;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Allocate "
            "memory for the context\n");

    if (groupId < 0) {
        context = (DMAN3_Context)MEM_calloc(_DMAN3_heapExt,
                sizeof(DMAN3_ContextObj), sizeof(Int));
    }
    else {
        /* Use the pre-allocated context object */
        context = _DMAN3_groupContext(groupId);
    }

    if (context == MEM_ILLEGAL) {
        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Could "
                "not allocate memory for the context\n");
        status = DMAN3_EOUTOFMEMORY;
    }
    else {
        /* Initialize the context object */
        context->groupId = groupId;
        context->refCount = 0;
        context->numTccs = 0;
        context->numPaRams = 0;
        context->tccBase = -1;  /* Initialize, in case maxTccs = 0 */

        maxTccs = nTccs;
        maxPaRams = nPaRams;

        if (groupId >= 0) {
            /*
             *  Allocate the maximum of nTccs and the number of TCCs
             *  configured for this groupId.
             */
            maxTccs = (_DMAN3_numTccGroup[groupId] > maxTccs) ?
                _DMAN3_numTccGroup[groupId] : maxTccs;

            /*
             *  Allocate the maximum of nPaRams and the number of PaRams
             *  configured for this groupId.
             */
            maxPaRams = (_DMAN3_numPaRamGroup[groupId] > maxPaRams) ?
                _DMAN3_numPaRamGroup[groupId] : maxPaRams;
        }

        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Allocate"
                " %d tccs\n", maxTccs);

        /* Allocate the TCCs */
        if ((maxTccs > 0) && !RMM_alloc(_DMAN3_rmmTccs, maxTccs,
                    (Uint32 *)&(context->tccBase))) {

            freeContext(context);

            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createContext> Exit"
                    " (status=DMAN3_OUTOFTCCS)\n");

            return (DMAN3_EOUTOFTCCS);
        }
        else {
            context->numTccs = maxTccs;
            context->nextFreeTccIdx = context->tccBase;

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Next "
                    "free tcc index %d\n", context->nextFreeTccIdx);

        }

        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Allocate"
                " %d paRams\n", maxPaRams);

        /* Allocate the PaRam */
        if (!RMM_alloc(_DMAN3_rmmPaRams, maxPaRams, (Uint32 *)&paRamBaseIndex)){

            freeContext(context);

            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createContext> Exit"
                    " (status=DMAN3_OUTOFPARAMS)\n");

            return (DMAN3_EOUTOFPARAMS);
        }
        else {
            context->numPaRams = maxPaRams;
            context->paRamBaseIdx = paRamBaseIndex; /* For freeing later */

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "createContext> Next"
                    "paRam index %d\n", context->paRamBaseIdx);

        }

        /* If successful allocation of TCCs and PaRam... */
#ifndef _DMAN3_CPUCOPY_
        DBC_assert(_DMAN3_paRamAddr != NULL);
#endif
        context->paRamBase = (Uns *)((Uns)_DMAN3_paRamAddr +
                (paRamBaseIndex * PARAMSIZEMAUS));
        context->nextFreePaRamAddr = context->paRamBase;
        context->numAssignedPaRams = 0;

        context->nextQdmaChannel = 0;
        *pContext = context;
    }

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "createContext> Exit "
            "(status=%d)\n", status);

    return (status);
}


/*
 *  ======== freeChannels ========
 *  Free memory for channel handles in dmaTab[].
 */
static Void freeChannels(IDMA3_ChannelRec dmaTab[], Int numChan)
{
    Int             i;
    IDMA3_Handle    chan;

    GT_2trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "freeChannels> Enter "
            "(dmaTab=0x%x, numChan=%d)\n", dmaTab, numChan);

    for (i = 0; i < numChan; i++) {
        if ((chan = dmaTab[i].handle) != MEM_ILLEGAL) {

            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "_freeChannels> "
                    "Freeing channel %d\n", chan);

            DMAN3_freeChannels(&chan, 1);
            dmaTab[i].handle = NULL;
        }
    }

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "freeChannels> Exit\n");
}


/*
 *  ======== freeContext ========
 *  Free context handle object that was used to start allocating from a fresh
 *  scratch allocation context.
 */
static Void freeContext(DMAN3_Context context)
{
#if DBC_ASSERTS
    Bool    fRet;
#endif

    DBC_require(context->refCount == 0);
    GT_assert(ti_sdo_fc_dman3_GTMask, context->refCount == 0);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "_freeContext> Enter "
            "(context=0x%x)\n", context);

    /* Free the TCCs */
    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "freeContext> Freeing "
            "%d tccs\n", context->numTccs);

    /* TODO:M should we 'invalidate' context->tccBase even if !DBC_ASSERTS? */
#if DBC_ASSERTS
    if (context->numTccs > 0) {
        fRet = RMM_free(_DMAN3_rmmTccs, context->tccBase, context->numTccs);
        context->tccBase = -1;
        DBC_assert(fRet);
    }
#else
    if (context->numTccs > 0) {
        RMM_free(_DMAN3_rmmTccs, context->tccBase, context->numTccs);
    }
#endif

    /* Free the PaRams */
    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "freeContext> Freeing "
            "%d params\n", context->numPaRams);

#if DBC_ASSERTS
    if (context->numPaRams > 0) {
        fRet = RMM_free(_DMAN3_rmmPaRams, context->paRamBaseIdx,
                context->numPaRams);
        context->paRamBaseIdx = -1;
        DBC_assert(fRet);
    }
#else
    if (context->numPaRams > 0) {
        RMM_free(_DMAN3_rmmPaRams, context->paRamBaseIdx, context->numPaRams);
    }
#endif


    if (context->groupId < 0) {
        MEM_free(_DMAN3_heapExt, context, sizeof(DMAN3_ContextObj));
    }

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "freeContext> Freeing "
            "context memory size 0x%x\n", sizeof(DMAN3_ContextObj));

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "freeContext> Exit\n");
}


/*
 *  ======== freeScratch ========
 */
/**
 *  Free any scratch memory in memTab[].
 */
static Void freeScratch(Int groupId, IALG_MemRec memTab[], Int numRecs)
{
    Int             i;

    GT_3trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "freeScratch> Enter "
            "(groupId=%d, memTab=0x%x, numRecs=%d)\n", groupId, memTab,
            numRecs);

    if (_DMAN3_freeScratch != NULL) {


        for (i = 0; i < numRecs; i++) {
            if ((memTab[i].attrs == IALG_SCRATCH) && (memTab[i].size > 0)) {

                GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "freeScratch> "
                        "Freeing scratch memory 0x%x of size 0x%x\n",
                        memTab[i].base, memTab[i].size);

                _DMAN3_freeScratch(groupId, memTab[i].base, memTab[i].size);
                memTab[i].base = MEM_ILLEGAL;
            }
        }
    }

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "freeScratch> Exit\n");

}


/*
 *  ======== getContext ========
 *  Get context handle for groupId. (In the future, we might have a linked
 *  list of contexts for a given groupId, with varying amounts of TCCs and
 *  PaRams. Therefore, we pass nTccs and nPaRams to this function).
 */
static Int getContext(DMAN3_Context *pContext, Int groupId, Int nTccs,
        Int nPaRams)
{
    Int           scratchId = groupId;
    DMAN3_Context context = NULL;
    Int status = DMAN3_SOK;

    GT_4trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "getContext> Enter "
            "(pContext=0x%x, groupId=%d, nTccs=%d, nPaRams=%d)\n", pContext,
            groupId, nTccs, nPaRams);

    *pContext = NULL;

    if (groupId >= 0) {
#ifdef xdc_target__isaCompatible_64P
        /*
         *  If allowUnshared is FALSE, don't allow channels to be created
         *  if the number of TCCs or PaRams needed is more than the number
         *  configured for the given scratch group.
         */
        if (DMAN3_PARAMS.allowUnshared == FALSE) {
            if (_DMAN3_numTccGroup[groupId] < nTccs) {
                return (DMAN3_ETCCCONFIG);
            }

            if (_DMAN3_numPaRamGroup[groupId] < nPaRams) {
                return (DMAN3_EPARAMCONFIG);
            }
        }
#endif
        context = _DMAN3_groupContext(groupId);

        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "getContext> Context "
                "exists for groupId %d, context 0x%x\n",groupId, context);

        /*
         *  Need to allocate a new context if one exists for the given
         *  groupId, but does not have enough PaRam or TCCs.
         *  Note: For non-64P targets where group contexts are pre-defined
         *  (and therefore never NULL), we need to check that the context
         *  reference count is > 0 before accessing the other fields.
         */
        if (context && (context->refCount > 0) && (
                (nPaRams > context->numPaRams) ||
                (nTccs > context->numTccs))) {
            /*
             *  The context for this groupId has been created with too few
             *  tccs or PaRams, so it cannot be used for this channel
             *  allocation. In this case, we will need to allocate a new
             *  context which will not share its tccs and PaRams.
             */
            GT_3trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                    "getContext> "
                    "Existing context has insufficient tccs and PaRams: "
                    "params: %d, tccs: %d, refCount: %d\n",
                    context->numPaRams, context->numTccs, context->refCount);

            groupId = -1;
            context = NULL;
        }
    }

    /*
     *  Create the context handle if not using any sharing group, or if
     *  using a sharing group but the context for that group has not yet
     *  been allocated, or, if not enough TCCs or PaRams are available.
     */
    if ((context == NULL) || (context->refCount == 0)) {

        GT_3trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "getContext> Creating "
                "context for group %d Tccs %d PaRams %d\n", groupId, nTccs,
                nPaRams);

        status = createContext(&context, groupId, nTccs, nPaRams);
        if (status != DMAN3_SOK) {
            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "getContext> Exit "
                    "(status=%d)\n", status);
            return (status);

        }
        context->scratchId = scratchId;
    }

    DBC_assert(context != NULL);

    /*
     *  Re-initialize next available TCC and PaRam back to the beginning.
     *  The channels allocated in this call will not share TCCs, PaRam.
     */
    context->nextFreeTccIdx = context->tccBase;
    context->nextFreePaRamAddr = context->paRamBase;
    context->numAssignedPaRams = 0;
    *pContext = context;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "getContext> Exit "
            "(status=DMAN3_SOK)\n");

    return (DMAN3_SOK);
}


/*
 *  ======== initHandle ========
 *  Initialize the IDMA3_Handle handle with resource allocation/channel state.
 */
static Bool initHandle(IDMA3_Handle handle, IDMA3_ChannelRec *dmaTab,
        Uint8 *tccList, Uns qdmaChannel, Int16 envSize, Void *env,
        Void *scratchEnv, Int seg, DMAN3_Context context)
{
    Bool          fRet = TRUE;

    GT_6trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "initHandle> Enter "
            "(handle=0x%x, dmaTab=0x%x, qdmaChannel=%d, env=0x%x, "
            "scratchEnv=0x%x, context=0x%x)\n", handle, dmaTab, qdmaChannel,
            env, scratchEnv, context);

    handle->numTccs = dmaTab->numWaits;
    handle->numPaRams = dmaTab->numTransfers;
    handle->qdmaChan  = qdmaChannel;
    handle->env  = env;

    GT_4trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "initHandle> Initializing "
            "IDMA3 handle with %d tccs, %d PaRams, qdmaChannel %d, env 0x%x\n",
            handle->numTccs, handle->numPaRams, handle->qdmaChan, env);

    if (env) {

        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "initHandle> Non-zero "
                "env handle\n");

        DBC_assert(envSize > 0);
        *(Uint32 *)handle->env = (Uint32)scratchEnv;
    }

    handle->protocol = dmaTab->protocol;

    /* Keep these for when we free the channel */
    ((DMAN3_Channel)handle)->context = context;
    ((DMAN3_Channel)handle)->sizeEnv = envSize;
    ((DMAN3_Channel)handle)->segEnv = seg;

    /*
     *  Assign Physical PARAM Blocks:
     *  Grant "numTransfers" many PARAM blocks, but initialize handle with the
     *  address of the "last" PARAM block.
     */
    handle->paRamAddr = ((Uns *)context->nextFreePaRamAddr) +
        (PARAMSIZE * (dmaTab->numTransfers - 1));

    /* Note: TccTable is provided by the framework. */
    handle->tccTable = tccList;

    /* until first transfer gets started there is no transfer to wait. */
    handle->transferPending = 0;

    GT_2trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "initHandle> paRam block "
            "0x%x, tcc Table 0x%x\n", handle->paRamAddr, handle->tccTable);

    /* Call the protocol's init function. */
    if (dmaTab->protocol && dmaTab->protocol->initHandle) {

        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "initHandle> Call "
                "protocol Init function\n");

        fRet = dmaTab->protocol->initHandle(handle);
    }

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "initHandle> Exit "
            "(status=%d)\n", fRet);

    return (fRet);
}

Int finalizeDMAN3() 
{

#ifdef _MULTI_PROCESS_
    /* Enter critical section */
    if (_DMAN3_lock != NULL) {
        LockMP_acquire(_DMAN3_lock);
    }
#endif

    if (--_DMAN3_refCount == 0) {
        /*
         *  The global DMAN3 reference count is 0, ie, no other processes
         *  are using DMAN3.
         */
        /* No need to free RMM handles, since they are just offsets */

#ifdef xdc_target__os_Linux
        freeDman3Obj();
#else
#ifdef _YOUR_TARGET_
        /*
         *  TODO: Free the block allocated in DMAN3_initImpl() for
         *  _DMAN3_data
         */
#else
        /* Free _DMAN3_data */
        MEM_free(_DMAN3_heapExt, _DMAN3_data, shmSize);
#endif
#endif
        shmFreeBase = shmBase = NULL;
    }
    else {
            /* Only this process's refCount has gone to 0. */
#ifdef xdc_target__os_Linux
            /* Detach from the shared memory, unmap DMA registers */
            unMapShm();
#endif
#ifdef _YOUR_TARGET_
            /* TODO: Any cleanup that may be needed at this point */
#endif
            shmFreeBase = shmBase = NULL;
    }

#ifdef _MULTI_PROCESS_
    /* Exit critical section */
    if (_DMAN3_lock != NULL) {
        LockMP_release(_DMAN3_lock);

            /* Delete the lock */
            LockMP_delete(_DMAN3_lock);
            _DMAN3_lock = NULL;
    }
#endif

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "DMAN3_exit> Exit\n");

    return (DMAN3_SOK);

}

Int initializeDMAN3() 
{
#ifndef _DMAN3_CPUCOPY_
    volatile Uns   *qchmap;
    Uns            *paRam;
#endif
    RMM_Attrs       rmmAttrs;
    Int             i;
    Int             rmmSize;
    Int             numTccs;
    UInt32          maxId = 0x0;
    UInt32          tempNum = 0x0;
    UInt32          chipTccs = 0x0;
    UInt32          cccfg = 0x0;
    
    heapInt = DMAN3_PARAMS.heapInternal;
    heapExt = DMAN3_PARAMS.heapExternal;

    heapInt = (heapInt == -1) ? heapExt : heapInt;
    heapExt = (heapExt == -1) ? heapInt : heapExt;
    
    /*
     *  First, need to know how many TCCs are used.
     */
    numTccs = getNumAssignedTccs(DMAN3_PARAMS.tccAllocationMaskH, &tempNum);
        
    /*
     *  Check if this allocation is legal    
     */

    /* 
     * Calculate the maximum Id of the tcc allocated. What is returned is 
     * the highest (Tcc number +1) that was configured for DMAN3  */

    if (tempNum > 0) {

        /* Adjust this number by 32 since this is the "H" mask */
        maxId = (tempNum-1) + 32;
    }
        
    numTccs += getNumAssignedTccs(DMAN3_PARAMS.tccAllocationMaskL,&tempNum);

    /* 
     * If the H mask was zero (and hence maxId was 0), then use this
     * returned number to figure out maxId
     */ 
    if (maxId == 0) {
        maxId = (tempNum-1);
    }

    /*
     *  We need to compute the size of the memory chunk that will hold
     *  _DMAN3_data and RMM objects. To compute the amount of memory
     *  needed by RMM, we can do 'dummy' creates of the RMM objects that
     *  will manage the TCCs and PaRams.
     */

    rmmSize = 2 * sizeof(RMM_Obj) + sizeof(RMM_Header) *
            ((DMAN3_PARAMS.numPaRamEntries + 1) / 2 + (numTccs + 1) / 2);

#ifdef _MULTI_PROCESS_

    /* Create a lock for protecting DMAN3 object */
    _DMAN3_lock = LockMP_create(_DMAN3_LOCKID);

    if (_DMAN3_lock == NULL) {
       GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
              "_DMAN3_initImpl> Failed to create IPC lock, key = 0x%x\n",
              _DMAN3_LOCKID);
       GT_assert(ti_sdo_fc_dman3_GTMask, _DMAN3_lock != NULL);
    }

#endif

#ifdef xdc_target__os_Linux
    /* Allocate the _DMAN3_data object for Linux */
    if (!getDman3Obj()) {
        /* Abort */
    }

#ifndef _DMAN3_CPUCOPY_
    /* Map the DMA registers */
    memfd = open("/dev/mem", O_RDWR | O_SYNC);
    _DMAN3_paRamAddr = mmap(0, PARAMREGSIZE, PROT_READ | PROT_WRITE,
           MAP_SHARED, memfd, (off_t)DMAN3_PARAMS.qdmaPaRamBase);

    _DMAN3_edma3Addr = mmap(0, EDMA3REGSIZE,
           PROT_READ | PROT_WRITE, MAP_SHARED, memfd,
           (off_t)((UInt32)DMAN3_PARAMS.qdmaPaRamBase - PARAMOFFSET));
#endif

#else
#ifdef _YOUR_TARGET_
    /*
     *  TODO: Allocate a block of memory for the _DMAN3_data object and
     *  other data that will be placed after the _DMAN3_data object. The
     *  size should be at least:
     *      sizeof(_DMAN3_Data) + rmmSize.
     *
     *  TODO: Set shmSize to the size of the allocated block.
     *  TODO: Set shmBase to the base address of the allocated block.
     *
     *  TODO: Set _DMAN3_data to the beginning of the allocated block,
     *        but make sure it is aligned on a 32 bit boundary.
     *
     *  TODO: Set shmFreeBase to the address of the first word after
     *        _DMAN3_data (see code below for an example).
     */

    /*
     *  TODO: Initialize _DMAN3_paRamAddr to the virtual or physical
     *  address of DMAN3_PARAMS.qdmaPaRamBase.
     */

    /*
     *  TODO: Initialize _DMAN3_edma3Addr to the virtual or physical
     *  address of (UInt32)DMAN3_PARAMS.qdmaPaRamBase - PARAMOFFSET.
     */
#else
    /*
     *  Allocate a block of memory for the _DMAN3_data object and
     *  other data that will be placed after the _DMAN3_data object. The
     *  size should be at least:
     *      sizeof(_DMAN3_Data) + rmmSize
     *  but we will add a little extra to be safe.
     */
    shmSize = sizeof(_DMAN3_Data) + rmmSize + 32;
    _DMAN3_data = (_DMAN3_Data *)MEM_calloc(DMAN3_PARAMS.heapExternal,
           shmSize, 0);

    /* Set shmBase to the base address of the allocated block. */
    shmBase = (Void *)_DMAN3_data;

    /* Make sure the shared memory base is aligned */
    shmFreeBase = (Void *)((UInt32)shmBase + ALIGN32(sizeof(_DMAN3_Data)));

    /* Start address of PaRams */
    _DMAN3_paRamAddr = DMAN3_PARAMS.qdmaPaRamBase;

    /* Start address of EDMA3 registers */
    _DMAN3_edma3Addr = (UInt32 *)((UInt32)DMAN3_PARAMS.qdmaPaRamBase -
           PARAMOFFSET);

#endif
#endif /* #ifdef xdc_target__os_Linux */

    cccfg =  *(((volatile Uns *)_DMAN3_edma3Addr)+1);
    chipTccs = ( (( ((cccfg)>>8)&0x7)==0) ? 0 : 
           (0x4 << (((cccfg)>>8) & 0x7)) );
    if (maxId >= chipTccs) {

        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_6CLASS,
               "_DMAN3_initImpl> DMAN3 tccAllocationMaskH configures tcc "
               "%d, that is more than available on hardware %d\n", 
               maxId , numTccs);

        GT_assert(ti_sdo_fc_dman3_GTMask, 0);
 
    }

#ifdef _MULTI_PROCESS_
    /* Enter critical section */
    LockMP_acquire(_DMAN3_lock);
#endif

    if (_DMAN3_refCount++ == 0) {
        /* Initialize to _DMAN3_data object */

#ifndef xdc_target__isaCompatible_64P
        /* Scratch env not supported for non-DSP targets */
        _DMAN3_allocScratch = NULL;
        _DMAN3_freeScratch = NULL;
#else
        /*
         *  Configured function for allocating IDMA3 object's env from
         *  shared scratch memory.
         */
        _DMAN3_allocScratch = DMAN3_PARAMS.scratchAllocFxn;

        /*
         *  Configured function for allocating IDMA3 object's env from
         *  shared scratch memory.
         */
        _DMAN3_freeScratch = DMAN3_PARAMS.scratchFreeFxn;

        /*
         *  If one of _DMAN3_allocScratch or _DMAN3_freeScratch is NULL,
         *  then the other must be NULL.
         */
        DBC_require((_DMAN3_allocScratch != NULL) ||
                    (_DMAN3_freeScratch == NULL));
        DBC_require((_DMAN3_freeScratch != NULL) ||
                    (_DMAN3_allocScratch == NULL));
        GT_assert(ti_sdo_fc_dman3_GTMask, (_DMAN3_allocScratch != NULL) ||
                    (_DMAN3_freeScratch == NULL));
        GT_assert(ti_sdo_fc_dman3_GTMask, (_DMAN3_freeScratch != NULL) ||
                    (_DMAN3_allocScratch == NULL));
#endif

        /*
         * Initialize the tccAllocationTable with the provided TCCs.
         * Use the TCCs assigned in the High range (32-63) first, then
         * Low (0-31) when initializing the _DMAN3_tccAllocationTable.
         * Update the total number of assigned TCC count
         * (_DMAN3_numAssignedTccs)
         */
        _DMAN3_numAssignedTccs = 0;
        updateTccAllocationTable(DMAN3_PARAMS.tccAllocationMaskH, 32);
        updateTccAllocationTable(DMAN3_PARAMS.tccAllocationMaskL, 0);

        /*
         *  Initialize _DMAN3_heapInt and _DMAN3_heapExt with provided
         *  heaps. Use internal heap for _DMAN3_heapInt when provided.
         *  (Actually this is a fatal error if ACPY3 uses internal DMA
         *  (IDMA) to write to PARAMs.
         */
        _DMAN3_heapInt = heapInt;
        _DMAN3_heapExt = heapExt;

        _DMAN3_heapIdma3 = (DMAN3_PARAMS.idma3Internal) ?
                    heapInt : heapExt;

    
        for (i = 0; i < DMAN3_MAXGROUPS; i++) {
            _DMAN3_numTccGroup[i] = DMAN3_PARAMS.numTccGroup[i];
            _DMAN3_numPaRamGroup[i] = DMAN3_PARAMS.numPaRamGroup[i];
        }

        /* Initialize _DMAN3_data->context[] to offsets in shared memory */
        for (i = 0; i < DMAN3_MAXGROUPS; i++) {
            _DMAN3_data->context[i] = OFFSET(_DMAN3_data,
                    &(_DMAN3_data->contextObjs[i]));
        }

        /*
         *  Attributes of TCC array that will be managed by RMM for
         *  allocation:
         *    segid - Use the external memory for the RMM object.
         *    base = 0 - We allocate indices of the array, starting with 0
         *    length = _DMAN3_numAssignedTccs - the number of TCCs in the array
         *    maxFreeListBlocks - Set to maximum number of free (unallocated)
         *        blocks that the array can fragment into. This happens when
         *        every other index is free.
         */
        rmmAttrs.segid = _DMAN3_heapExt;
        rmmAttrs.base = 0;
        rmmAttrs.length = _DMAN3_numAssignedTccs;
        rmmAttrs.maxFreeListBlocks = (_DMAN3_numAssignedTccs + 1) / 2;
        rmmAttrs.allocFxn = shmAlloc;
        rmmAttrs.freeFxn = shmFree;

        /*
         *  Can't put a pointer in shared memory area if it's shared by
         *  different processes. Therefore, we store the offset from the
         *  _DMAN3_data.
         */
        _DMAN3_data->rmmTccs = OFFSET(_DMAN3_data, RMM_create(&rmmAttrs));

        DBC_assert(_DMAN3_rmmTccs != MEM_ILLEGAL);
        GT_assert(ti_sdo_fc_dman3_GTMask, _DMAN3_rmmTccs != MEM_ILLEGAL);

        /*
         *  Attributes of PaRam entry numbers that will be managed by RMM for
         *  allocation:
         *    segid - Use the external memory for the RMM object.
         *    base - The first PaRam index for DMAN3.
         *    length - Number of PaRam entries for DMAN3.
         *    maxFreeListBlocks - Set to maximum number of free (unallocated)
         *        blocks that can PaRam indices can fragment into. This
         *        happens when every other index is free.
         */
        rmmAttrs.base = DMAN3_PARAMS.paRamBaseIndex;
        rmmAttrs.length = DMAN3_PARAMS.numPaRamEntries;
        rmmAttrs.maxFreeListBlocks = (DMAN3_PARAMS.numPaRamEntries + 1) / 2;

        _DMAN3_data->rmmPaRams = OFFSET(_DMAN3_data, RMM_create(&rmmAttrs));

        DBC_assert(_DMAN3_rmmPaRams != MEM_ILLEGAL);
        GT_assert(ti_sdo_fc_dman3_GTMask, _DMAN3_rmmPaRams != MEM_ILLEGAL);

         _DMAN3_nullPaRam = DMAN3_PARAMS.nullPaRamIndex;
         GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                    "_DMAN3_initImpl> Null PaRam is %d\n", _DMAN3_nullPaRam);

#ifndef _DMAN3_CPUCOPY_
        /* Set qchmap to address of first QCHMAP register */
        qchmap = (volatile Uns *)((Uns)_DMAN3_edma3Addr + QCHMAPOFFSET);

        /*
         *  Set the trigger word of each QDMA channel to 7, so that when we
         *  set the static bit (of opts field of PaRam which is word 0) of the
         *  null PaRam, we don't start a transfer, in case the QCHMAP register
         *  happened to have the null PaRam with trigger word 0.
         * Note:-
         *  Even though all channels might not be owned by DMAN3, we are
         *  setting the static bit of ALL QDMA channels that are mapped to
         *  paRam 0. This is because QDMA channels on bring up point to paRam 0
         *  by default. Even if they aren't owned by DMAN3, not changing their
         *  trigger words, might cause extraneous transfers when setting the
         *  static bit of the OPT field of paRam 0.
         *  Since paRam 0 is owned by DMAN3, ideally no QDMA channels belonging
         *  to other frameworks should point to PaRam 0
         */
        for (i = 0; i < DMAN3_PARAMS.maxQdmaChannels; i++) {
            if (_DMAN3_nullPaRam == ((*qchmap >> 5 ) & 0x1FF)) {
                GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS,
                        "_DMAN3_initImpl> Qdma channel %d is pointing to "
                        "the nullPaRam. Change its trigger word bit to 0x7 so "
                        "it doesn't get triggered when we modify the null "
                        "PaRam\n", i);
                *qchmap = *qchmap | (0x7 << 2);
            }
            qchmap++;
        }

        /*
         *  Set the static bit of opts field of the null PaRam, since
         *  QCHMAP may get set to this PaRam when the channel is
         *  activated/deactivated. (The opts field is the first word of the
         *  PaRam entry).
         */
        paRam = (Uns *)((Uns)_DMAN3_paRamAddr +
                (_DMAN3_nullPaRam * PARAMENTRYSIZE * sizeof(UInt32)));

        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_2CLASS, "_DMAN3_initImpl> "
               "Setting static bit of null paRam (address = 0x%x)\n", paRam);
        *paRam |= 0x8;
#endif

        if (FALSE == checkDMAN3Configuration(_DMAN3_numAssignedTccs)) {
           GT_0trace(ti_sdo_fc_dman3_GTMask, GT_6CLASS, "_DMAN3_initImpl> "
                   "DMAN3 SOC-specific configuration different from hardware, "
                   "continuing with configured values\n");
        }

        initQdmaConfiguration();
    }

#ifdef _MULTI_PROCESS_
    /* Exit critical section */
    LockMP_release(_DMAN3_lock);
#endif

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "_DMAN3_initImpl> "
            "Exiting, _DMAN3_data = 0x%x\n", _DMAN3_data);

    return (DMAN3_SOK);
}



/*
 *  ======== getNumAssignedTccs ========
 */
static Int getNumAssignedTccs(UInt32 mask32b, UInt32 * maxTcc)
{
    Int numTccs = 0;
    Int i;

    *maxTcc = 0x0;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "getNumAssignedTccs> Enter\n");

    for (i = 0; i < 32; i++) {
        if (mask32b & (0x1 << i)) {
            numTccs++;
            *maxTcc = i+1; 
        }
    }
    
    
    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_4CLASS, "getNumAssignedTccs> Max TCC"
            " is 0x%x\n", *maxTcc);

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "getNumAssignedTccs> Exit\n");

    return (numTccs);
}


/* Functions for RMM to alloc and free its objects from shared memory */

/*
 *  ======== shmAlloc ========
 */
/* ARGSUSED - this line tells the compiler not to warn about unused args. */
static Void *shmAlloc(Int segid, UInt32 size, UInt32 align)
{
    UInt32  nFreeBytes;
    Void   *base;
    UInt32  alnSize;

    GT_3trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "DMAN3 shmAlloc> segid = %d, size = 0x%x, align = 0x%x\n",
            segid, size, align);

    /*
     *  Don't need to handle align parameter (yet). Just align size on
     *  4-byte boundary.
     */
    alnSize = ALIGN32(size);

    /* Number of bytes remaining in the shared memory chunk */
    nFreeBytes = shmSize + (UInt32)shmBase -(UInt32)shmFreeBase;

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "DMAN3 shmAlloc> bytes remaining before allocation: 0x%x\n",
            nFreeBytes);

    if (alnSize > nFreeBytes) {
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "DMAN3 shmAlloc> Shared memory allocation of [0x%x] bytes "
                "failed. Remaining free bytes: [0x%x]\n", alnSize, nFreeBytes);
        return (NULL);
    }

    base = shmFreeBase;
    shmFreeBase = (Void *)((UInt32)shmFreeBase + alnSize);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "DMAN3 shmAlloc> bytes remaining after allocation: 0x%x\n",
            shmSize + (UInt32)shmBase - (UInt32)shmFreeBase);

    memset((void *)base, 0, size);

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "DMAN3 shmAlloc> returning base address: 0x%x\n", base);

    return (base);
}

/*
 *  ======== shmFree ========
 */
/* ARGSUSED - this line tells the compiler not to warn about unused args. */
static Bool shmFree(Int segid, Void *addr, UInt32 size)
{
    return (TRUE);
}

/*
 *  ======== updateTccAllocationTable ========
 */
static Void updateTccAllocationTable(UInt32 mask32b, Int base)
{
    Int i;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "updateTccAllocationTable> "
            "Enter\n");

    for (i = 0; i < 32; i++) {

        if (mask32b & (0x1 << i)) {
            _DMAN3_tccAllocationTable[_DMAN3_numAssignedTccs++] = base + i;
        }
    }

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "updateTccAllocationTable> Exit\n");
}

/*
 * ======== checkDMAN3Configuration ========
 */
/* ARGSUSED */
static Bool checkDMAN3Configuration(Int tccs)
{
    Bool status = TRUE;
#ifndef _DMAN3_CPUCOPY_
    Uns cccfg = 0x0;
    short numQdma = 0;
    short numParam = 0;
    short numTC = 0;
    volatile Uns * edma3Base = (Uns *)_DMAN3_edma3Addr;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "checkDMAN3Configuration> Enter\n");

    cccfg = edma3Base[1];
    cccfg = (edma3Base[1] >> 4);

    numQdma = (cccfg & 0x7) << 0x1;

    if  (DMAN3_PARAMS.maxQdmaChannels != numQdma) {
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_6CLASS,
                "checkDMAN3Configuration> DMAN3 max QDMA channels configured "
                "to be %d, hardware reads %d\n", DMAN3_PARAMS.maxQdmaChannels,
                numQdma);

        status = FALSE;
    }

    cccfg = (cccfg >> 4);
    cccfg = (cccfg >> 4);
    numParam = (((cccfg & 0x7) == 0) ? 0x0 : (0x10 << (cccfg & 0x7)));

    if  (DMAN3_PARAMS.maxPaRamEntries != numParam) {
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_6CLASS,
                "checkDMAN3Configuration> DMAN3 max PaRam entries configured "
                "to be %d, hardware reads %d\n", DMAN3_PARAMS.maxPaRamEntries,
                numParam);

        status = FALSE;
    }

    cccfg = (cccfg>>4);
    numTC = (cccfg &0x7) + 1;

    if  (DMAN3_PARAMS.maxTCs != numTC) {
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_6CLASS,
                "checkDMAN3Configuration> DMAN3 max TCs configured to be %d, "
                "hardware reads %d\n", DMAN3_PARAMS.maxTCs, numTC);

        status = FALSE;
    }
#endif

    GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "checkDMAN3Configuration> Exit (status=%d)\n", status);

    return (status);
}

/*
 *  ======== initQdmaConfiguration ========
 */
static Void initQdmaConfiguration()
{
#ifndef _DMAN3_CPUCOPY_
    Int i,chan;
    UInt32 * addr;
    Uns queueNum;
    Uns qeesr;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "_initQdmaConfiguration> Enter\n");

    /* Event queue to TC channel mapping */
    addr = (UInt32 *)((UInt32)_DMAN3_edma3Addr + QUETCMAPOFFSET);
    for (i = 0 ; i < DMAN3_PARAMS.maxTCs; i++) {
        (*addr) |= (DMAN3_PARAMS.queueTCMap[i] << (i *4));
    }

    /* Set Priority for each of the event queues. */
    addr = (UInt32 *)((UInt32)_DMAN3_edma3Addr + QUEPRIOFFSET);
    for (i = 0; i < DMAN3_PARAMS.maxTCs; i++) {
        (*addr) |= ( DMAN3_PARAMS.queuePri[i] << (i * 4));
    }

    /* QDMA region access enable register 0 */
    addr = (UInt32 *)((UInt32)_DMAN3_edma3Addr + QRAE0OFFSET);
    (*addr) = 0xFF;

    /* Set QDMA to queue mapping */
    addr = (UInt32 *)((UInt32)_DMAN3_edma3Addr + QDMAQNUMOFFSET);
    for (i = 0; i < DMAN3_PARAMS.numQdmaChannels; i++) {
        chan = DMAN3_PARAMS.qdmaChannels[i];
        queueNum = DMAN3_PARAMS.qdmaQueueMap[i];
        (*addr) |= (queueNum << (4 * chan));
    }

    /* Queue Event Enable Set register */
    qeesr = 0x0;
    for (i = 0; i < DMAN3_PARAMS.numQdmaChannels; i++) {
        qeesr |= (0x1 << DMAN3_PARAMS.qdmaChannels[i]);
    }
    addr = (UInt32 *)((UInt32)_DMAN3_edma3Addr + QEESROFFSET);
    (*addr) = qeesr;

    GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
            "_initQdmaConfiguration> Exit\n");

#endif /* DMAN3_CPUCOPY */
}

#ifdef xdc_target__os_Linux

/*
 *  ======== freeDman3Obj ========
 */
static Void freeDman3Obj()
{
#ifndef _DMAN3_CPUCOPY_
    /* close memfd, unmap */
    if (munmap(_DMAN3_edma3Addr, EDMA3REGSIZE) == -1) {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to unmap _DMAN3_edma3Addr [0x%x]\n",
                _DMAN3_edma3Addr);
    }

    if (munmap(_DMAN3_paRamAddr, PARAMREGSIZE) == -1) {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to unmap _DMAN3_paRamAddr [0x%x]\n",
                _DMAN3_paRamAddr);
    }

    close(memfd);
#endif
    /* Detach the shared memory */
    if (shmdt(shmBase) == -1) {
        /* error */
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to detach shared memory [0x%x], key = 0x%x\n",
                shmBase, _DMAN3_SHMID);
    }

    /* Free the shared memory */
    if (shmctl(shmId, IPC_RMID, 0) == -1) {
        /* error */
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to free shared memory [0x%x], key = 0x%x\n",
                shmBase, _DMAN3_SHMID);
    }
}

/*
 *  ======== getDman3Obj ========
 */
static Bool getDman3Obj()
{
    Bool            shmExists = TRUE;
    Int             lockRefCount;

#ifdef _MULTI_PROCESS_
    /* Enter critical section */
    LockMP_acquire(_DMAN3_lock);
#endif

    /*
     *  Allocate or locate the chunk of shared memory where internal
     *  DMAN3 objects will be placed. This is to ensure that multiple
     *  processes will see the same DMAN3 objects.
     *  First attempt to get the shared memory without the IPC_CREAT
     *  flag, to see if it's already been created.
     */
    shmSize = SHMSIZE;
    shmId = shmget((key_t)(_DMAN3_SHMID), SHMSIZE, 0666);
    if (shmId == -1) {
        /*
         *  Failed to get the shared memory, so it probably hasn't been
         *  created yet.
         */
        shmId = shmget((key_t)(_DMAN3_SHMID), SHMSIZE, 0666 | IPC_CREAT);
        if (shmId == -1) {
            /* Couldn't create the shared memory */
            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                    "_DMAN3_initImpl> Failed to create shared memory "
                    "region, key = 0x%x\n", _DMAN3_SHMID);

#ifdef _MULTI_PROCESS_
            /* Exit critical section */
            LockMP_release(_DMAN3_lock);
#endif
            /* Abort*/
            GT_assert(ti_sdo_fc_dman3_GTMask, shmId != -1);
            return (FALSE);
        }
        else {
            /* The segment had not been created before */
            GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                    "_DMAN3_initImpl> Created shared memory, key = 0x%x\n",
                    _DMAN3_SHMID);
            shmExists = FALSE;
        }
    }
    else {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                "_DMAN3_initImpl> Shared memory, key = 0x%x already"
                " exists\n", _DMAN3_SHMID);
    }

    /* Get the address of the shared memory region with shmat() */
    shmBase = shmat(shmId, NULL, 0);
    if (shmBase == (Void *)-1) {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "_DMAN3_initImpl> Failed to get shared memory address, "
                "key = 0x%x\n", _DMAN3_SHMID);
        shmBase = NULL;

#ifdef _MULTI_PROCESS_
        /* Exit critical section */
        LockMP_release(_DMAN3_lock);
#endif
        /* Abort */
        GT_assert(ti_sdo_fc_dman3_GTMask, shmBase != NULL);
        return (FALSE);
    }

    /* Make sure the shared memory base is aligned */
    shmFreeBase = (Void *)ALIGN32(shmBase);

    _DMAN3_data = shmFreeBase;

    /* Make sure the shared memory base is aligned */
    shmFreeBase = (Void *)((UInt32)shmBase + ALIGN32(sizeof(_DMAN3_Data)));

    if (shmExists) {
        /*
         *  Shared memory segment already exists. Check the DMAN3 lock
         *  reference count. Since the lock is created only once per
         *  process, if the lock reference count is 1, the calling process
         *  is the only one using DMAN3 global data. In that case, a
         *  process must have died, and will re-initialize the DMAN3
         *  global data.
         */
        lockRefCount = LockMP_getRefCount(_DMAN3_lock);

        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_ENTER, "_DMAN3_initImpl> "
                "DMAN3 shared memory reference count = %d\n"
                "DMAN3 lock reference count = %d\n",
                _DMAN3_refCount, lockRefCount);

        if (lockRefCount == 1) {
            GT_2trace(ti_sdo_fc_dman3_GTMask, GT_5CLASS,
                    "_DMAN3_initImpl> Warning: Inconsistent reference"
                    " count between DMAN3 shared memory [%d] and lock"
                    " [%d]\n", _DMAN3_refCount, lockRefCount);
            GT_0trace(ti_sdo_fc_dman3_GTMask, GT_5CLASS,
                    "_DMAN3_initImpl> Warning: DMAN3 lock reference "
                    "count indicates no other processes are "
                    "currently using DMAN3 shared memory.  "
                    "Re-initializing DMAN3 shared memory\n");

            /* This will cause the DMAN3 data to be re-initialized */
            shmExists = FALSE;
        }
    }

    if (!shmExists) {
        /* Initialize the reference count in the shared data area */
        GT_0trace(ti_sdo_fc_dman3_GTMask, GT_ENTER,
                "Initializing DMAN3 shared data\n");
        _DMAN3_refCount = 0;

        /* Initialize the _DMAN3_data object to 0's */
        memset((Void *)_DMAN3_data, 0, sizeof(_DMAN3_Data));
    }

#ifdef _MULTI_PROCESS_
    /* Exit critical section */
    LockMP_release(_DMAN3_lock);
#endif
    return (TRUE);
}

/*
 *  ======== unMapShm ========
 */
static Void unMapShm()
{
#ifndef _DMAN3_CPUCOPY_
    /* close memfd, unmap */
    if (munmap(_DMAN3_edma3Addr, EDMA3REGSIZE) == -1) {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to unmap _DMAN3_edma3Addr [0x%x]\n",
                _DMAN3_edma3Addr);
    }

    if (munmap(_DMAN3_paRamAddr, PARAMREGSIZE) == -1) {
        GT_1trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to unmap _DMAN3_paRamAddr [0x%x]\n",
                _DMAN3_paRamAddr);
    }

    close(memfd);
#endif
    /*
     *  Don't move this block of code. Cannot detach shmBase until
     *  _DMAN3_refCount is checked.
     */
    /* Detach the shared memory, but don't free it. */
    if (shmdt(shmBase) == -1) {
        /* error */
        GT_2trace(ti_sdo_fc_dman3_GTMask, GT_7CLASS,
                "Failed to detach shared memory [0x%x], key = 0x%x\n",
                shmBase, _DMAN3_SHMID);
    }
}

#endif
/*
 *  @(#) ti.sdo.fc.dman3; 1, 0, 4,278; 11-8-2009 20:57:20; /db/atree/library/trees/fc/fc-l04x/src/
 */

